#!/usr/bin/python
# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Franois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: exiftool.py,v 1.2 2003/07/10 19:09:52 jfroche Exp $'

## ===========================================================================
##  Exiftool is freeware by Robert F. Tobler <rft@cg.tuwien.ac.at>.
##  No warrenties whatsoever on the usefulness for whatever purpose.
##  However if the tool is useful to you, you have improvements or
##  suggestions, or if you just like it, I would appreciate feedback.
## ===========================================================================
##  NAME:       exiftool
##  TYPE:       python script
##  CONTENT:    decode EXIF header in JPEGs from digital cameras
##  VERSION:    1.1
## ===========================================================================
##  AUTHORS:    rft     Robert F. Tobler <rft@cg.tuwien.ac.at>
## ===========================================================================
##  HISTORY:
##
##  31-Jul-01 11:23:48  rft     added byte-swap (DImage 7), E995 MakerNote
##  10-May-00 14:12:42  rft     created
## ===========================================================================
##  MANUAL:
##
##  NAME
##      exiftool, decode EXIF headers in JPEGs from digital cameras
##
##  SYNOPSIS
##      exiftool [options] <jpeg-1> <jpeg-2> ...
##
##  DESCRIPTION
##      Most digital cameras store additional information about each image
##      directly within each JPEG file. This inforation is located in the
##      so-called EXIF-header and contains such things as exposure time,
##      f-stop, aso. This tool decodes this information and prints the
##      informat, stores it in a HTML file, or renames the jpeg file
##      according to the date and time the image was taken.
##
##  OPTIONS
##      -d              Print debug output. This is most useful together with
##                      verbose output (-v). It reports the offset of each
##                      directory and data entry within the EXIF header.
##
##      -h              A short help page.
##
##      -i              Prints the tags of each JPEG according to a text
##                      template file that can be specified with the -t
##                      option.
##
##      -n              No action. Do not rename files or generate HTML
##                      files, just report intended actions.
##
##      -m              Print this manual.
##
##      -o <directory>  Specify the output directory where renamed files
##                      or HTML files should be placed. In this case, the
##                      rename option will actually copy the files, leaving
##                      the originals intact.
##
##      -r              Rename files according to exif date
##
##      -t <file>       Take file contents as a template for either HTML
##                      files (if the file has an extension of '.html',
##                      '.HTML', '.htm', or '.HTM') or as a template for
##                      the info text used by the -i option otherwise.
##                      Within the template, the value of a tag can be
##                      printed by using the following syntax:
##
##                              %(<tagname>)s
##
##                      e.g. %(Model)s will report the camera model which
##                      produced the JPEG file. For a list of the tagnames
##                      that can be used, you can use the -v option.
##
##      -v              Verbose output. This reports all EXIF tags that are
##                      found in each JPEG.
##
##      -w              Writes a HTML file for each JPEG according to a
##                      template file that can be specified with the -t
##                      option.
##
##  AUTHOR
##      Robert F. Tobler <rft@cg.tuwien.ac.at>
##
## ===========================================================================

import sys
import os
import string
import struct
import time
import shutil
from stat import *

## ---------------------------------------------------------------------------
##  The whole parser is map-driven. The map takes the tag number as key and
##  returns the name and the interpretation of the tag. There are a number
##  of standard interpretations: 'FLOAT', 'EXPOSURE', 'BIAS'. For short-
##  values a special interpretation can be directly stated as a map. A
##  key in a special interpretation can also be made vendor specific, by
##  specifiying the pair (vendor, key) as a key.
## ---------------------------------------------------------------------------

## ---------------------------------------------------------------------------
##  Nikon 990 MakerNote Tags http://members.tripod.com/~tawba/990exif.htm
## ---------------------------------------------------------------------------
NIKON_99x_MAKERNOTE = {
    	0x0001:	('MN_0x0001',	    	    	None, {}),
	0x0002:	('MN_ISOSetting',   	    	None, {}),
	0x0003:	('MN_ColorMode',    	    	None, {}),
	0x0004:	('MN_Quality',	    	    	None, {}),
	0x0005:	('MN_Whitebalance', 	    	None, {}),
	0x0006:	('MN_ImageSharpening',	    	None, {}),
	0x0007:	('MN_FocusMode',    	    	None, {}),
    	0x0008:	('MN_FlashSetting', 	    	None, {}),
	0x000A:	('MN_0x0008',	    	    	None, {}),
	0x000F:	('MN_ISOSelection', 	    	None, {}),
	0x0080:	('MN_ImageAdjustment',	    	None, {}),
	0x0082:	('MN_AuxiliaryLens',	    	None, {}),
	0x0085:	('MN_ManualFocusDistance',  	'FLOAT', {}),
	0x0086:	('MN_DigitalZoomFactor',    	'FLOAT', {}),
	0x0088:	('MN_AFFocusPosition',	    	'MAP', {
		    '\00\00\00\00': 'Center',
		    '\00\01\00\00': 'Top',
		    '\00\02\00\00': 'Bottom',
		    '\00\03\00\00': 'Left',
		    '\00\04\00\00': 'Right',
		}),
	0x008f:	('MN_0x008f',     	    	None, {}),
	0x0094:	('MN_0x0094',     	    	None, {}),
	0x0095:	('MN_NoiseReduction',     	None, {}), ## guess
	0x0010:	('MN_DataDump',     	    	None, {}),

	0x0011:	('MN_0x0011',     	    	None, {}),
	0x0e00:	('MN_0x0e00',     	    	None, {}),
}

TAG_INFO = {
	0x00fe:	('NewSubFileType',  	    	None, {}),
	0x0100:	('ImageWidth',      	    	None, {}),
	0x0101:	('ImageLength',     	    	None, {}),
	0x0102:	('BitsPerSample',   	    	None, {}),
	0x0103:	('Compression',     	    	None, {}),
	0x0106:	('PhotometricInterpretation',	None, {}),
	0x010e:	('ImageDescription', 	    	None, {}),
	0x010f:	('Make',    	    	    	None, {}),
	0x0110:	('Model',   	    	    	None, {}),
	0x0111:	('StripOffsets',    	    	None, {}),
	0x0112:	('Orientation',     	    	None, {}),
	0x0115:	('SamplesPerPixel', 	    	None, {}),
	0x0116:	('RowsPerStrip',    	    	None, {}),
	0x0117:	('StripByteCounts', 	    	None, {}),
	0x011a:	('XResolution',     	    	None, {}),
	0x011b:	('YResolution',     	    	None, {}),
	0x011c:	('PlanarConfiguration',     	None, {}),
	0x0128:	('ResolutionUnit',  	    	'MAP',	{
			1:	'Not Absoulute',
			2:	'Inch',
			3:	'Centimeter'
		}),
	0x0131:	('Software',	    	    	None, {}),
	0x0132:	('DateTime',	    	    	None, {}),
	0x013b:	('Artist',  	    	    	None, {}),
	0x0142:	('TileWidth',	    	    	None, {}),
	0x0143:	('TileLength',	    	    	None, {}),
	0x0144:	('TileOffsets',     	    	None, {}),
	0x0145:	('TileByteCounts',  	    	None, {}),
	0x014a:	('SubIFDs', 	    	    	None, {}),
	0x015b:	('JPEGTables',	    	    	None, {}),
	0x0201:	('JPEGInterchangeFormat',   	None, {}), # from EXIFread
	0x0202:	('JPEGInterchangeFormatLength', None, {}), # from EXIFread
	0x0211:	('YCbCrCoefficients',	    	None, {}),
	0x0212:	('YCbCrSubSampling',	    	None, {}),
	0x0213:	('YCbCrPositioning',	    	None, {}),
	0x0214:	('ReferenceBlackWhite',     	None, {}),
	0x828f:	('BatteryLevel',    	    	None, {}),
	0x8298:	('Copyright',	    	    	None, {}),
	0x829a:	('ExposureTime',    	    	'EXPOSURE', {}),
	0x829d:	('FNumber', 	    	    	'FLOAT', {}),
	0x83bb:	('IPTC_NAA',	    	    	None, {}),
	0x8773:	('InterColorProfile',	    	None, {}),
	0x8822:	('ExposureProgram', 	    	'MAP', {
			0:	'Unidentified',
			1:	'Manual',
			2:	'Program Normal',
			3:	'Aperture Priority',
			4:	'Shutter Priority',
			5:	'Program Creative',
			6:	'Program Action',
			7:	'Portrait Mode',
			8:	'Landscape Mode',

		}),
	0x8824:	('SpectralSensitivity',     	None, {}),
	0x8825:	('GPSInfo', 	    	    	None, {}),
	0x8827:	('ISOSpeedRatings', 	    	None, {}),
	0x8828:	('OECF',    	    	    	None, {}),
	0x8829:	('Interlace',	    	    	None, {}),
	0x882a:	('TimeZoneOffset',  	    	None, {}),
	0x882b:	('SelfTimerMode',   	    	None, {}),
	0x8769:	('ExifOffset',	    	    	None, {}), ## from EXIFread
	0x9000: ('ExifVersion',     	    	None, {}), ## from EXIFread
	0x9003: ('DateTimeOriginal',	    	None, {}),
	0x9004: ('DateTimeDigitized',	    	None, {}), ## from EXIFread
	0x9101: ('ComponentsConfiguration', 	None, {}), ## from EXIFread
	0x9102:	('CompressedBitsPerPixel',  	None, {}),
	0x9201:	('ShutterSpeedValue',	    	'SPEED', {}),
	0x9202:	('ApertureValue',   	    	'FLOAT', {}),
	0x9203:	('BrightnessValue', 	    	None, {}),
	0x9204:	('ExposureBiasValue',	    	'BIAS', {}),
	0x9205:	('MaxApertureValue',	    	'FLOAT', {}),
	0x9206:	('SubjectDistance', 	    	None, {}),
	0x9207:	('MeteringMode',    	    	'MAP', {
			0:		    'Unidentified',
			1:		    'Average',
			2:		    'CenterWeightedAverage',
			3:		    'Spot',
			4:		    'MultiSpot',
			('NIKON', '', 5):   'Matrix',   # vendor specific
		 }),
	0x9208:	('LightSource',     	    	None, {}),
	0x9209:	('Flash',   	    	    	'MAP', {
			0:		'no',
			1:		'fired',
			5:		'fired (?)', # no return sensed
			7:		'fired (!)', # return sensed
			9:		'fill fired',
			13:		'fill fired (?)',
			15:		'fill fired (!)',
			16:		'off',
			24:		'auto off',
			25:		'auto fired',
			29:		'auto fired (?)',
			31:		'auto fired (!)',
			32:		'not available'
		}),
	0x920a:	('FocalLength',     	    	'FLOAT', {}),
	0x920b:	('FlashEnergy',     	    	None, {}),
	0x920c:	('SpatialFrequencyResponse',	None, {}),
	0x920d:	('Noise',   	    	    	None, {}),
	0x920e:	('FocalPlaneXResolution',   	None, {}),
	0x920f:	('FocalPlaneYResolution',   	None, {}),
	0x9210:	('FocalPlaneResolutionUnit',	None, {}),
	0x9211:	('ImageNumber',     	    	None, {}),
	0x9212:	('SecurityClassification',  	None, {}),
	0x9213:	('ImageHistory',    	    	None, {}),
	0x9214:	('SubjectLocation', 	    	None, {}),
	0x9215:	('ExposureIndex',   	    	None, {}),
	0x9216:	('TIFF_EPStandardID',	    	None, {}),
	0x9217:	('SensingMethod',   	    	None, {}),
	0x927c: ('MakerNote',	    	    	'MAKE_MODEL', {
	    	    	('NIKON', 'E990'):  ('TAGS', NIKON_99x_MAKERNOTE),
	    	    	('NIKON', 'E995'):  ('TAGS', NIKON_99x_MAKERNOTE),
	    	    	('',''):	    ( None, {} ),
	    	}),

	0xa002: ('ImageWidth',	    	    	None, {}), # guess by rft
	0xa003: ('ImageHeight',     	    	None, {}), # guess by rft
}

TAG_TYPE_INFO = {
	1:	('BYTE',	1),
	2:	('ASCII',	1),
	3:	('SHORT',	2),
	4:	('LONG',	4),
	5:	('RATIO',	8),
	6:	('SBYTE',	1),
	7:	('UNDEF',	1),
	8:	('SSHORT',	2),
	9:	('SLONG',	4),
	10:	('SRATIO',	8),
	11:	('FLOAT',	4),
	12:	('DOUBLE',	8)
}

SHORT_0 = 0
SHORT_1 = 1
LONG_0 = 0
LONG_1 = 1
LONG_2 = 2
LONG_3 = 3

def set_little_endian():
    global SHORT_0
    global SHORT_1
    global LONG_0
    global LONG_1
    global LONG_2
    global LONG_3
    SHORT_0 = 1
    SHORT_1 = 0
    LONG_0 = 3
    LONG_1 = 2
    LONG_2 = 1
    LONG_3 = 0

def short_value(str):
    return (ord(str[SHORT_1])<<8)|ord(str[SHORT_0])

def long_value(str):
    return (ord(str[LONG_3])<<24)|(ord(str[LONG_2])<<16)| \
    	    (ord(str[LONG_1])<<8)|ord(str[LONG_0])

def exif_seek(file, offset):
    file.seek(offset + 12)

def format_bytes(str):
    sequence = []
    for i in range(0, len(str)):
	sequence.append(ord(str[i]))
    return `sequence`

def parse_tag(file, tags, exif_info):
    tag_id = short_value(file.read(2))
    tag_type = short_value(file.read(2))
    tag_count = long_value(file.read(4))
    verbose_opt = 0
    debug_opt = 0
    (tag_name, tag_format, tag_map) = \
    	    exif_info.get(tag_id, ("Tag0x%x" % tag_id, None, {}))

    if tag_format == 'MAKE_MODEL':
    	make = tags.get('Make','')
	model = tags.get('Model','')
	if tag_map.has_key((make,model)):
    	    (tag_format, tag_map) = tag_map[(make,model)]
	elif tag_map.has_key((make,'')):
	    (tag_format, tag_map) = tag_map[(make,'')]
	else:
	    (tag_format, tag_map) = tag_map[('','')]

    (tag_type_name, tag_type_len) = TAG_TYPE_INFO[tag_type]

    if verbose_opt:
     
	    print "%30s:" % tag_name,
	    if debug_opt: print "%6s %3d" % (tag_type_name, tag_count),

    tag_len = tag_type_len * tag_count

    pos = file.tell()
    if tag_len > 4:
	tag_offset = long_value(file.read(4))
	if verbose_opt and debug_opt: print "@%03x" % tag_offset,
	exif_seek(file, tag_offset)
    else:
    	if verbose_opt and debug_opt: print "    ",

    if verbose_opt and debug_opt: print ":",

    if tag_type == 1 or tag_type == 6 or tag_type == 7: # (S)BYTE, UNDEFINED
	if tag_count == 1:
	    tags[tag_name] = `ord(file.read(1)[0])`
	else:
	    if tag_format == 'TAGS':	    	## a tag contains a directory
		num_entries = short_value(file.read(2))
		if verbose_opt: print num_entries, 'tags'
		for field in range(0, num_entries):
		    parse_tag(file, tags, tag_map)
	    else:
		str = file.read(tag_count)
		if tag_format == 'MAP':
		    if tag_map.has_key(str):
			tags[tag_name] = tag_map[str]
		    else:
			make = tags['Make']
			model = tags['Model']
			if tag_map.has_key((make,model,str)):
		    	    tags[tag_name] = tag_map[(make,model,str)]
			elif tag_map.has_key((make,'',str)):
		    	    tags[tag_name] = tag_map[(make,'',str)]
			else:
		    	    tags[tag_name] = format_bytes(str)
		else:
		    tags[tag_name] = format_bytes(str)

    if tag_type == 2: # ASCII
	str = file.read(tag_count-1)
	tags[tag_name] = string.strip(str)

    if tag_type == 3 or tag_type == 8: # (S)SHORT
	for i in range(0, tag_count):
	    short = short_value(file.read(2))
	    if tag_format == 'MAP':
		if tag_map.has_key(short):
		    tags[tag_name] = tag_map[short]
		else:
		    make = tags['Make']
		    model = tags['Model']
		    if tag_map.has_key((make,model,short)):
		    	tags[tag_name] = tag_map[(make,model,short)]
		    elif tag_map.has_key((make,'',short)):
		    	tags[tag_name] = tag_map[(make,'',short)]
		    else:
		    	tags[tag_name] = `short`
	    else:
		tags[tag_name] = `short`
    if tag_type == 4 or tag_type == 9: # (S)LONG
	for i in range(0, tag_count):
	    long = long_value(file.read(4))
	    tags[tag_name] = `long`

    if tag_type == 5 or tag_type == 10: # (S)RATIONAL
	for i in range(0, tag_count):
	    long1 = long_value(file.read(4))
	    long2 = long_value(file.read(4))
	    if tag_format == 'BIAS':
		if long1 > 0:
		    val = "+%3.1f" % (long1*1.0/long2)
		elif long1 < 0:
		    val = "-%3.1f" % (-long1*1.0/long2)
		else:
		    val = "+/-0.0"
		tags[tag_name] = val
	    elif tag_format == 'FLOAT':
	    	if long2 == 0:
		    tags[tag_name] = "0"
		else:
		    tags[tag_name] = "%g" % (long1*1.0/long2)
	    elif tag_format == 'EXPOSURE':
		if long1 >= long2:
		    tags[tag_name] = "%g" % (long1*1.0/long2)
		else:
		    if long1 > 1:
			tags[tag_name] = "1/%d" % ((2*long2+long1)/(2*long1))
		    else:
		    	tags[tag_name] = "1/%d" % long1
	    elif tag_format == 'SPEED':
	    	if long2 == 0:
		    tags[tag_name] = "0"
		elif long1 > long2:
		    tags[tag_name] = "1/%g" % pow(2, long1*1.0/long2)
		else:
		    tags[tag_name] = "%g" % pow(0.5, long1*1.0/long2)
	    else:
		tags[tag_name] = "%d/%d" % (long1, long2)

    file.seek(pos+4)

    if verbose_opt:
    	if tags.has_key(tag_name): print tags[tag_name],
	print

## ---------------------------------------------------------------------------
def exif_parse(path_name):
    file = open(path_name, "rb")
    skip = file.read(12)    	    	    # skip the jpeg header
    order = file.read(2)    	    	    # tiff order magic

    if order == 'MM':
    	set_little_endian()

    code = file.read(2)     		    # tiff magic 42
    verbose_opt = 0
    debug_opt = 0
    if len(code) == 2 and short_value(code) == 42:
	(dir_name, file_name) = os.path.split(path_name)
	(photo_name, ext) = os.path.splitext(file_name)

#    	if verbose_opt:
#	    print "-------------"*(5+debug_opt)
#	    print "                     PhotoName: %s" % photo_name

	tags = { 'PhotoName': photo_name }

	offset = long_value(file.read(4))

	while offset > 0:

	    exif_seek(file,offset)
	    num_entries = short_value(file.read(2))
	    if verbose_opt and debug_opt:
	    	print "                              :",
		print "       %3d"% num_entries,
	    	print "@%03x" % offset

	    for field in range(0, num_entries):
		parse_tag(file, tags, TAG_INFO)

	    offset = long_value(file.read(4))

    	    if offset == 0: 	    	    	    # special handling to get
	    	if tags.has_key('ExifOffset'):	    # exif directory
		    offset = string.atoi(tags['ExifOffset'])
		    del tags['ExifOffset']
		if tags.has_key('MakerNote'): # we don't want the builder notes
		    del tags['MakerNote']
                for key in tags.keys():
			if(key.find('Tag0')!=-1):
				del tags[key]	

    else:
	tags = {}

    file.close()
    return tags

## ---------------------------------------------------------------------------
def print_manual_and_exit():
    script = open(sys.argv[0],'r')
    line = script.readline()
    while (line[0:11] != '##  MANUAL:'):
        line = script.readline()
    line = script.readline()
    while (line[0:11] != '## ========'):
        if line[0:2] == '##':
            sys.stdout.write('  ' + line[2:])
        else:
            sys.stdout.write(line)
        line = script.readline()
    script.close()
    sys.exit(0)

## ---------------------------------------------------------------------------
def write_file_format(file_name, format, map):
    file = open(file_name, "w")
    file.write(format % map)
    file.close()

## ---------------------------------------------------------------------------
def read_file(path):
    try:
	file = open(path, "rb")
	data = file.read()
	file.close()
    except IOError:
    	print "ERROR: could not read file '%s'!" % path_name
	sys.exit(-1)
    return data

## ---------------------------------------------------------------------------
##  In order to print the exif header with a % format string, even when some
##  of the tags are missing, we use a PrintMap which returns 'unknown' for
##  all missing tags.
## ---------------------------------------------------------------------------
class PrintMap:
    def __init__(self, map):
    	self.map = map
    def __getitem__(self, key):
    	val = self.map.get(key)
	if val == None:
	    val = 'unknown'
    	return val

## ---------------------------------------------------------------------------
##  This function creates a filename from the date as it is read in the
##  exif header. In Nikon and Sony jpegs the format is: yyyy:mm:dd hh:mm:ss.
## ---------------------------------------------------------------------------
def date_name(date_str):
    return  date_str[:4] +'.'+ date_str[5:7] +'.'+ date_str[8:10] +'-'+\
    	    date_str[11:13] +'.'+ date_str[14:16] +'.'+ date_str[17:19]

## ---------------------------------------------------------------------------
##  'TEXT_FORMAT'
##  	This is the default template for printing EXIF information with the
##  	-i option.
## ---------------------------------------------------------------------------
TEXT_FORMAT = """__________________________________________
        Photo Name: %(PhotoName)s
            Camera: %(Make)s %(Model)s
              Date: %(DateTime)s
     Exposure Time: %(ExposureTime)s sec
          Aperture: f/%(FNumber)s
  Exposure Program: %(ExposureProgram)s
     Exposure Bias: %(ExposureBiasValue)s
     Metering Mode: %(MeteringMode)s
             Flash: %(Flash)s
       FocalLength: %(FocalLength)s mm
"""

## ---------------------------------------------------------------------------
##  'HTML_FORMAT'
##  	This is the default template for generating HTML files containing
##  	EXIF information with the -w option.
## ---------------------------------------------------------------------------
HTML_FORMAT = """<html>
<head>
    <title>%(PhotoName)s / Info</title>
</head>
<body bgcolor="#000000" text="#DDDDDD" link="#0077FF" vlink="#00BBFF">
<h2>%(PhotoName)s</h2>
<table align=left border=1 cellspacing=1>
    <tr><td align=right>Camera</td>
	<td align=left>%(Make)s %(Model)s</td></tr>
    <tr><td align=right>Date</td>
	<td align=left>%(DateTime)s</td></tr>
    <tr><td align=right>Exposure Time</td>
	<td align=left>%(ExposureTime)s sec</td></tr>
    <tr><td align=right>Aperture</td>
	<td align=left>f/%(FNumber)s</td></tr>
    <tr><td align=right>Exposure Program</td>
	<td align=left>%(ExposureProgram)s</td></tr>
    <tr><td align=right>Exposure Bias</td>
	<td align=left>%(ExposureBiasValue)s</td></tr>
    <tr><td align=right>Metering Mode</td>
	<td align=left>%(MeteringMode)s</td></tr>
    <tr><td align=right>Flash</td>
	<td align=left>%(Flash)s</td></tr>
    <tr><td align=right>Focal Length</td>
	<td align=left>%(FocalLength)s mm</td></tr>
</table>
</body>
</html>
"""

## ---------------------------------------------------------------------------
def print_usage_and_exit():
    print "USAGE:  exiftool [opts] <jpg-1> ... <jpg-n>"
    print "OPTS:   -h           this help page"
    print "        -m           print manual page. use 'exiftool -m | less'"
    print "        -d           debug output"
    print "        -i           print info for each jpeg file"
    print "        -n           no action"
    print "        -o <dir>     specify output directory; copy not rename"
    print "        -r           rename files according to exif date"
    print "        -t <file>    text or html template file (extension!)"
    print "        -v           verbose output, prints all tags"
    print "        -w           write a html file for each jpeg file"
    print "NOTE:   You have to specify at least one of: -i -r -w -v !"
    print "        If no JPEGs are specified, all JPEGs in the current",
    print "dir are taken."
    sys.exit(-1)

## ---------------------------------------------------------------------------
#try:
#
#    global debug_opt
#    global verbose_opt
#
#    help_opt	= 0
#    debug_opt	= 0
#    verbose_opt = 0
#    write_opt	= 0
#    rename_opt	= 0
#    info_opt	= 0
#    noact_opt	= 0
#    outdir_par	= ''
#
#    file_list = []
#
#    act = 0
#    argc = len(sys.argv)
#    i = 0
#    while i+1 < argc:
#    	i = i+1
#	arg = sys.argv[i]
#    	if arg == '-h':     print_usage_and_exit()
#    	if arg == '-m':     print_manual_and_exit()
#	if arg == '-d':     debug_opt = 1 ; 	    	    	continue
#	if arg == '-v':     verbose_opt = 1 ;	act = 1 ;    	continue
#    	if arg == '-w':     write_opt = 1 ; 	act = 1 ; 	continue
#    	if arg == '-r':     rename_opt = 1 ; 	act = 1 ;   	continue
#    	if arg == '-i':     info_opt = 1 ; 	act = 1 ;   	continue
#    	if arg == '-n':     noact_opt = 1 ; 	    	    	continue
#    	if arg == '-o':
#	    if i+1 < argc: i = i+1 ; outdir_par = sys.argv[i] ; continue
#	    print "ERROR: no directory for option '-d'!"
#	    sys.exit(-1)
#	if arg == '-t':
#	    if i+1 < argc:
#	    	i = i+1
#		t_par = sys.argv[i]
#		(t_name, t_ext) = os.path.splitext(t_par)
#		if  t_ext == '.html' or t_ext == '.HTML' \
#		     or t_ext == '.htm' or t_ext == '.HTM':
#		    HTML_FORMAT = read_file(t_par)
#		else:
#		    TEXT_FORMAT = read_file(t_par)
#		continue
#	    print "ERROR: no template file for option '-t'!"
#	    sys.exit(-1)
#	if arg[0] == '-':
#	    print "ERROR: unknown option '%s'!" % arg
#	    sys.exit(-1)
#    	file_list.append(arg)
#
#    if outdir_par and not os.path.isdir(outdir_par):
#    	print "ERROR: '%s' is not directory!" % outdir_par
#	sys.exit(-1)
#
#    if not act: print_usage_and_exit()
#
#    if len(file_list) == 0:
#    	file_list = os.listdir('.')
#
#    for path_name in file_list:
#	(dir_name, jpeg_name) = os.path.split(path_name)
#	(photo_name, ext) = os.path.splitext(jpeg_name)
#	if not os.path.exists(path_name):
#	    print path_name, 'UNREADABLE_FILE'
#	    continue
#	if ext == '.jpg' or ext == '.JPG' or ext == '.jpeg' or ext == '.JPEG':
#	    tags = exif_parse(path_name)
#	    if tags == {}:
#	    	print path_name, 'NO_EXIF_HEADER'
#		continue
#	    if rename_opt:
#	    	new_name = date_name(tags['DateTime'])
#		print path_name,
#		if new_name == '0000.00.00-00.00.00':
#		    print 'NO_VALID_TIMESTAMP'
#		else:
#		    new_jpeg = new_name + '.jpg'
#    	    	    if outdir_par:
#			new_path_name = os.path.join(outdir_par, new_jpeg)
#		    else:
#			new_path_name = os.path.join(dir_name, new_jpeg)
#		    if not noact_opt:
#		    	tags['PhotoName'] = new_name
#			new_path_noext = new_path_name[:-4]+'-'
#			count = 1
#			while new_path_name and os.path.exists(new_path_name):
#			    jpeg1 = read_file(path_name)
#			    jpeg2 = read_file(new_path_name)
#			    if jpeg1 == jpeg2:
#			    	new_path_name = ''
#				print 'IDENTICAL_FILE_EXISTS',
#			    else:
#			    	new_path_name = new_path_noext+`count`+'.jpg'
#				count = count + 1
#				if debug_opt:
#				    print '[file exists, incremented name]',
#			if new_path_name:
#			    if outdir_par:
#				shutil.copyfile(path_name, new_path_name)
#			    else:
#				os.rename(path_name, new_path_name)
#		    print new_path_name
#    	    print_tags = PrintMap(tags)
#	    if info_opt:
#	    	print TEXT_FORMAT % print_tags
#	    if write_opt:
#	        html_name = photo_name + '.html'
#		if outdir_par:
#		    html_name = os.path.join(outdir_par, html_name)
#		print path_name, html_name
#		if not noact_opt:
#	    	    write_file_format(html_name, HTML_FORMAT, print_tags)
#
#
#except Exception, error:
#    if isinstance(error, IOError) and str(error) == '[Errno 32] Broken pipe':
#    	sys.exit(-1)
#    if isinstance(error,KeyboardInterrupt):
#	sys.exit(-1)
#    raise
### ==========================================================================


